import kotlinx.coroutines.test.runTest
import net.sergeych.crypto2.initCrypto
import net.sergeych.kiloparsec.*
import net.sergeych.kiloparsec.adapter.acceptTcpDevice
import net.sergeych.kiloparsec.adapter.connectTcpDevice
import kotlin.random.Random
import kotlin.test.Test
import kotlin.test.assertEquals
import kotlin.test.assertIs

class TcpTest {
    class TestException : Exception("test1")

    @Test
    fun tcpTest() = runTest {
        initCrypto()
        //        Log.connectConsole(Log.Level.DEBUG)
data class Session(
    var data: String,
)

        val port = 27170 + Random.nextInt(1, 200)

val cmdSave by command<String, Unit>()
val cmdLoad by command<Unit, String>()
val cmdDrop by command<Unit, Unit>()
val cmdException by command<Unit, Unit>()

        val cli = KiloInterface<Session>().apply {
            registerError { TestException() }
            onConnected { session.data = "start" }
            on(cmdSave) { session.data = it }
            on(cmdLoad) {
                session.data
            }
            on(cmdException) {
                throw TestException()
            }
            on(cmdDrop) {
                throw LocalInterface.BreakConnectionException()
            }
        }
        val server = KiloServer(cli, acceptTcpDevice(port)) {
            Session("unknown")
        }

val client = KiloClient<Unit>() {
    addErrors(cli)
    // TODO: add register error variant
    connect { connectTcpDevice("localhost:$port") }
}

assertEquals("start", client.call(cmdLoad))

client.call(cmdSave, "foobar")
assertEquals("foobar", client.call(cmdLoad))

        val res = kotlin.runCatching { client.call(cmdException) }
        assertIs<TestException>(res.exceptionOrNull())
        assertEquals("foobar", client.call(cmdLoad))

        assertThrows<RemoteInterface.ClosedException> { client.call(cmdDrop) }

        // reconnect?
        assertEquals("start", client.call(cmdLoad))

        server.close()
    }
}